

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from App.Common import package_home
PACKAGE_HOME = package_home(globals())

from types import FunctionType as function

from Products.PolicyTestCase.PolicyTestCase import PolicyTestCase
from Products.PolicyTestCase import patches
from Products.PolicyTestCase.utils import makeSuiteFromCSV
from AccessControl import Unauthorized
from Products.CMFCore.utils import _checkPermission as checkPerm
from Products.CMFCore.permissions import AccessContentsInformation
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent, ListFolderContents



from Products.PloneTestCase import PloneTestCase

PloneTestCase.setupPloneSite()

class NoMatchingRole(Exception):
    """the specified role doesn't match any of the supported role"""

class TestDefaultPlone(PolicyTestCase):
    

    def afterSetUp(self):
        portal = self.portal
        wf = portal.portal_workflow
        self.loginAsPortalOwner()
        wf.setChainForPortalTypes(['Document', ], ['plone_workflow',])
        wf.setChainForPortalTypes(['Folder'], ['plone_workflow',])
        self.portal.invokeFactory('Folder', 'projects')
        self.portal.projects.invokeFactory('Folder', 'television')
        self.portal.projects.television.invokeFactory('Document', 'page1')
        container = self.portal.projects.television
        content = container.page1
        self.container_path = 'projects/television'
        self.content_path = 'page1'
        content_state = self.state['content_state']
        role = self.state['role']

        # state of content
        # check current state first to see if we need to move state
        current_content_state = wf.getInfoFor(content, 'review_state', '')
        if current_content_state != content_state:
            # move to desired state by looking up appropriate transition
            infos = wf.listActionInfos(object=content)
            for i in infos:
                if i['transition'].new_state_id == content_state:
                    wf.doActionFor(content, i['id'])
                    break

        # Init roles
        if role == 'anon':
            self.logout()
            return
        elif role == 'contributor':
            #self.portal.acl_users._doAddUser('contributor', 'secret',
            #                                 ['Member', 'Contributor'], [])
            self.portal.acl_users._doAddUser('contributor', 'secret',
                                             ['Member',], [])
            content.manage_setLocalRoles('contributor', ('Contributor', ))
            self.login('contributor')
        elif role == 'editor':
            self.portal.acl_users._doAddUser('editor', 'secret',
                                             ['Member', 'Editor'], [])
            self.login('editor')
        elif role == 'manager':
            self.portal.acl_users._doAddUser('manager', 'secret',
                                             ['Manager'], [])
            self.login('manager')
        elif role == 'member':
            self.portal.acl_users._doAddUser('member', 'secret', ['Member'], [])
            self.login('member')
        elif role == 'owner':
            self.portal.acl_users._doAddUser('owner', 'secret',
                                             ['Member'], [])
            content.manage_setLocalRoles('owner', ('Owner', ))
            self.login('owner')
        elif role == 'reader':
            self.portal.acl_users._doAddUser('reader', 'secret',
                                             ['Member', 'Reader'], [])
            self.login('reader')
        elif role == 'reviewer':
            self.portal.acl_users._doAddUser('reviewer', 'secret',
                                             ['Member', 'Reviewer'], [])
            self.login('reviewer')
        else:
            # need to raise an exception if the role is not specified
            # otherwise the test will run in any case and produce a wrong
            # result 
            raise NoMatchingRole
        content.reindexObject()


    def ViewContent(self):
        project = self.portal.unrestrictedTraverse(self.container_path)
        self.assertNotRaises(Unauthorized, project.restrictedTraverse,
                             self.content_path)


    def NoViewContent(self):
        project = self.portal.unrestrictedTraverse(self.container_path)
        self.assertRaises(Unauthorized, project.restrictedTraverse,
                          self.content_path)


    def EditContent(self):
        portal = self.portal
        project = portal.unrestrictedTraverse(self.container_path)
        self.assertNotRaises(Unauthorized, project.restrictedTraverse,
                             self.content_path)
        contentob = project.restrictedTraverse(self.content_path)

        self.failUnless(checkPerm(ModifyPortalContent, contentob))
        fti = contentob.getTypeInfo()
        aliases = fti.getMethodAliases()
        edit_method = aliases['edit']
        self.assertNotRaises(Unauthorized, getattr(contentob, edit_method))


    def NoEditContent(self):
        portal = self.portal
        project = portal.unrestrictedTraverse(self.container_path)
        try:
            contentob = project.restrictedTraverse(self.content_path)
        except Unauthorized:
            return

        try:
            fti = contentob.getTypeInfo()
            aliases = fti.getMethodAliases()
            edit_method = aliases['edit']
            self.assertRaises(Unauthorized, getattr(contentob, edit_method))
        except Unauthorized:
            # We might not be able to even see the object
            # hence can't edit it, this is ok
            pass


def test_suite():
    from unittest import TestSuite
    suite = TestSuite()
    csv = open('%s/test_community.csv' % PACKAGE_HOME)
    suite.addTest(makeSuiteFromCSV(TestDefaultPlone, csv))
    return suite


if __name__ == '__main__':
    framework()

